@@ -275,7 +275,7 @@ impl Utmpx {
             }
         }
 
-        Ok(host.to_string())
+        Ok(host)
     }
 
     /// Iterate through all the utmp records.
@@ -412,63 +412,63 @@ impl UtmpxRecord {
     /// A.K.A. ut.ut_type
     pub fn record_type(&self) -> i16 {
         match self {
-            UtmpxRecord::Traditional(utmpx) => utmpx.record_type(),
+            Self::Traditional(utmpx) => utmpx.record_type(),
             #[cfg(feature = "feat_systemd_logind")]
-            UtmpxRecord::Systemd(systemd) => systemd.record_type(),
+            Self::Systemd(systemd) => systemd.record_type(),
         }
     }
 
     /// A.K.A. ut.ut_pid
     pub fn pid(&self) -> i32 {
         match self {
-            UtmpxRecord::Traditional(utmpx) => utmpx.pid(),
+            Self::Traditional(utmpx) => utmpx.pid(),
             #[cfg(feature = "feat_systemd_logind")]
-            UtmpxRecord::Systemd(systemd) => systemd.pid(),
+            Self::Systemd(systemd) => systemd.pid(),
         }
     }
 
     /// A.K.A. ut.ut_id
     pub fn terminal_suffix(&self) -> String {
         match self {
-            UtmpxRecord::Traditional(utmpx) => utmpx.terminal_suffix(),
+            Self::Traditional(utmpx) => utmpx.terminal_suffix(),
             #[cfg(feature = "feat_systemd_logind")]
-            UtmpxRecord::Systemd(systemd) => systemd.terminal_suffix(),
+            Self::Systemd(systemd) => systemd.terminal_suffix(),
         }
     }
 
     /// A.K.A. ut.ut_user
     pub fn user(&self) -> String {
         match self {
-            UtmpxRecord::Traditional(utmpx) => utmpx.user(),
+            Self::Traditional(utmpx) => utmpx.user(),
             #[cfg(feature = "feat_systemd_logind")]
-            UtmpxRecord::Systemd(systemd) => systemd.user(),
+            Self::Systemd(systemd) => systemd.user(),
         }
     }
 
     /// A.K.A. ut.ut_host
     pub fn host(&self) -> String {
         match self {
-            UtmpxRecord::Traditional(utmpx) => utmpx.host(),
+            Self::Traditional(utmpx) => utmpx.host(),
             #[cfg(feature = "feat_systemd_logind")]
-            UtmpxRecord::Systemd(systemd) => systemd.host(),
+            Self::Systemd(systemd) => systemd.host(),
         }
     }
 
     /// A.K.A. ut.ut_line
     pub fn tty_device(&self) -> String {
         match self {
-            UtmpxRecord::Traditional(utmpx) => utmpx.tty_device(),
+            Self::Traditional(utmpx) => utmpx.tty_device(),
             #[cfg(feature = "feat_systemd_logind")]
-            UtmpxRecord::Systemd(systemd) => systemd.tty_device(),
+            Self::Systemd(systemd) => systemd.tty_device(),
         }
     }
 
     /// A.K.A. ut.ut_tv
     pub fn login_time(&self) -> time::OffsetDateTime {
         match self {
-            UtmpxRecord::Traditional(utmpx) => utmpx.login_time(),
+            Self::Traditional(utmpx) => utmpx.login_time(),
             #[cfg(feature = "feat_systemd_logind")]
-            UtmpxRecord::Systemd(systemd) => systemd.login_time(),
+            Self::Systemd(systemd) => systemd.login_time(),
         }
     }
 
@@ -477,27 +477,27 @@ impl UtmpxRecord {
     /// Return (e_termination, e_exit)
     pub fn exit_status(&self) -> (i16, i16) {
         match self {
-            UtmpxRecord::Traditional(utmpx) => utmpx.exit_status(),
+            Self::Traditional(utmpx) => utmpx.exit_status(),
             #[cfg(feature = "feat_systemd_logind")]
-            UtmpxRecord::Systemd(systemd) => systemd.exit_status(),
+            Self::Systemd(systemd) => systemd.exit_status(),
         }
     }
 
     /// check if the record is a user process
     pub fn is_user_process(&self) -> bool {
         match self {
-            UtmpxRecord::Traditional(utmpx) => utmpx.is_user_process(),
+            Self::Traditional(utmpx) => utmpx.is_user_process(),
             #[cfg(feature = "feat_systemd_logind")]
-            UtmpxRecord::Systemd(systemd) => systemd.is_user_process(),
+            Self::Systemd(systemd) => systemd.is_user_process(),
         }
     }
 
     /// Canonicalize host name using DNS
     pub fn canon_host(&self) -> IOResult<String> {
         match self {
-            UtmpxRecord::Traditional(utmpx) => utmpx.canon_host(),
+            Self::Traditional(utmpx) => utmpx.canon_host(),
             #[cfg(feature = "feat_systemd_logind")]
-            UtmpxRecord::Systemd(systemd) => systemd.canon_host(),
+            Self::Systemd(systemd) => systemd.canon_host(),
         }
     }
 }
