@@ -5,26 +5,25 @@
 
 // spell-checker:ignore (paths) atim sublink subwords azerty azeaze xcwww azeaz amaz azea qzerty tazerty tsublink testfile1 testfile2 filelist fpath testdir testfile
 // spell-checker:ignore selfref ELOOP smallfile
+
 #[cfg(not(windows))]
 use regex::Regex;
 
-use uutests::at_and_ucmd;
-use uutests::new_ucmd;
 #[cfg(not(target_os = "windows"))]
 use uutests::unwrap_or_return;
 use uutests::util::TestScenario;
 #[cfg(not(target_os = "windows"))]
 use uutests::util::expected_result;
-use uutests::util_name;
+use uutests::{at_and_ucmd, new_ucmd, util_name};
 
 #[cfg(not(target_os = "openbsd"))]
 const SUB_DIR: &str = "subdir/deeper";
 const SUB_DEEPER_DIR: &str = "subdir/deeper/deeper_dir";
 const SUB_DIR_LINKS: &str = "subdir/links";
 const SUB_DIR_LINKS_DEEPER_SYM_DIR: &str = "subdir/links/deeper_dir";
-#[cfg(not(target_os = "openbsd"))]
+#[cfg(all(not(target_os = "android"), not(target_os = "openbsd")))]
 const SUB_FILE: &str = "subdir/links/subwords.txt";
-#[cfg(not(target_os = "openbsd"))]
+#[cfg(all(not(target_os = "android"), not(target_os = "openbsd")))]
 const SUB_LINK: &str = "subdir/links/sublink.txt";
 
 #[test]
@@ -69,7 +68,11 @@ fn du_basics(s: &str) {
     assert_eq!(s, answer);
 }
 
-#[cfg(all(not(target_vendor = "apple"), not(target_os = "windows")))]
+#[cfg(all(
+    not(target_vendor = "apple"),
+    not(target_os = "windows"),
+    not(target_os = "openbsd")
+))]
 fn du_basics(s: &str) {
     let answer = concat!(
         "8\t./subdir/deeper/deeper_dir\n",
@@ -119,7 +122,8 @@ fn du_basics_subdir(s: &str) {
 #[cfg(all(
     not(target_vendor = "apple"),
     not(target_os = "windows"),
-    not(target_os = "freebsd")
+    not(target_os = "freebsd"),
+    not(target_os = "openbsd")
 ))]
 fn du_basics_subdir(s: &str) {
     // MS-WSL linux has altered expected output
@@ -447,7 +451,8 @@ fn du_d_flag(s: &str) {
 #[cfg(all(
     not(target_vendor = "apple"),
     not(target_os = "windows"),
-    not(target_os = "freebsd")
+    not(target_os = "freebsd"),
+    not(target_os = "openbsd")
 ))]
 fn du_d_flag(s: &str) {
     // MS-WSL linux has altered expected output
@@ -520,7 +525,8 @@ fn du_dereference(s: &str) {
 #[cfg(all(
     not(target_vendor = "apple"),
     not(target_os = "windows"),
-    not(target_os = "freebsd")
+    not(target_os = "freebsd"),
+    not(target_os = "openbsd")
 ))]
 fn du_dereference(s: &str) {
     // MS-WSL linux has altered expected output
@@ -1392,6 +1398,17 @@ fn test_du_files0_from_stdin_with_invalid_zero_length_file_names() {
         .stderr_contains("-:2: invalid zero-length file name");
 }
 
+#[test]
+fn test_du_files0_from_stdin_with_stdin_as_input() {
+    new_ucmd!()
+        .arg("--files0-from=-")
+        .pipe_in("-")
+        .fails_with_code(1)
+        .stderr_is(
+            "du: when reading file names from standard input, no file name of '-' allowed\n",
+        );
+}
+
 #[test]
 fn test_du_files0_from_dir() {
     let ts = TestScenario::new(util_name!());
