@@ -241,8 +241,7 @@ fn test_should_report_invalid_empty_number_on_blank_stdin() {
 
 #[test]
 fn test_suffixes() {
-    // TODO add support for ronna (R) and quetta (Q)
-    let valid_suffixes = ['K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y' /*'R' , 'Q'*/];
+    let valid_suffixes = ['K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y', 'R', 'Q', 'k'];
 
     for c in ('A'..='Z').chain('a'..='z') {
         let args = ["--from=si", "--to=si", &format!("1{c}")];
@@ -264,12 +263,12 @@ fn test_suffixes() {
 
 #[test]
 fn test_should_report_invalid_suffix_on_nan() {
-    // GNU numfmt reports this one as “invalid number”
+    // GNU numfmt reports this one as "invalid number"
     new_ucmd!()
         .args(&["--from=auto"])
         .pipe_in("NaN")
         .fails()
-        .stderr_is("numfmt: invalid suffix in input: 'NaN'\n");
+        .stderr_is("numfmt: invalid number: 'NaN'\n");
 }
 
 #[test]
@@ -700,7 +699,7 @@ fn test_invalid_stdin_number_with_warn_returns_status_0() {
         .pipe_in("4Q")
         .succeeds()
         .stdout_is("4Q\n")
-        .stderr_is("numfmt: invalid suffix in input: '4Q'\n");
+        .stderr_is("numfmt: rejecting suffix in input: '4Q' (consider using --from)\n");
 }
 
 #[test]
@@ -718,7 +717,7 @@ fn test_invalid_stdin_number_with_abort_returns_status_2() {
         .args(&["--invalid=abort"])
         .pipe_in("4Q")
         .fails_with_code(2)
-        .stderr_only("numfmt: invalid suffix in input: '4Q'\n");
+        .stderr_only("numfmt: rejecting suffix in input: '4Q' (consider using --from)\n");
 }
 
 #[test]
@@ -728,7 +727,7 @@ fn test_invalid_stdin_number_with_fail_returns_status_2() {
         .pipe_in("4Q")
         .fails_with_code(2)
         .stdout_is("4Q\n")
-        .stderr_is("numfmt: invalid suffix in input: '4Q'\n");
+        .stderr_is("numfmt: rejecting suffix in input: '4Q' (consider using --from)\n");
 }
 
 #[test]
@@ -737,7 +736,7 @@ fn test_invalid_arg_number_with_warn_returns_status_0() {
         .args(&["--invalid=warn", "4Q"])
         .succeeds()
         .stdout_is("4Q\n")
-        .stderr_is("numfmt: invalid suffix in input: '4Q'\n");
+        .stderr_is("numfmt: rejecting suffix in input: '4Q' (consider using --from)\n");
 }
 
 #[test]
@@ -753,7 +752,7 @@ fn test_invalid_arg_number_with_abort_returns_status_2() {
     new_ucmd!()
         .args(&["--invalid=abort", "4Q"])
         .fails_with_code(2)
-        .stderr_only("numfmt: invalid suffix in input: '4Q'\n");
+        .stderr_only("numfmt: rejecting suffix in input: '4Q' (consider using --from)\n");
 }
 
 #[test]
@@ -762,7 +761,7 @@ fn test_invalid_arg_number_with_fail_returns_status_2() {
         .args(&["--invalid=fail", "4Q"])
         .fails_with_code(2)
         .stdout_is("4Q\n")
-        .stderr_is("numfmt: invalid suffix in input: '4Q'\n");
+        .stderr_is("numfmt: rejecting suffix in input: '4Q' (consider using --from)\n");
 }
 
 #[test]
